/**********************************************************************
 *
 * z3950proto.cpp --
 * Copyright (C) 2000  The New Zealand Digital Library Project
 *
 * A component of the Greenstone digital library software
 * from the New Zealand Digital Library Project at the
 * University of Waikato, New Zealand.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *********************************************************************/

#include "z3950proto.h"
#include "comtypes.h"
#include <stdio.h>

/*** 
    We will use the "collection" argument as the name of the database
    to query on the z3950 server.
***/


z3950_server::z3950_server() {
  count=0;
  info=NULL;
  meta["iconcollection"]="/~johnmcp/gsdl/images/zserver.png";
}




// z3950 Protocol functions
z3950proto::z3950proto() {
  zserver_count=0;
}

void z3950proto::add_server (const z3950_server& zserver) {

  // append the new server
  zserver_count++;
  zservers.push_back(zserver);
}

void z3950proto::configure (const text_t &/*key*/, 
			    const text_tarray &/*cfgline*/) {

  // this is called for each line in the gsdlsite.cfg file
  /*  cerr << "z3950proto::configure called:"
      << "key is " << key.getcstr()
      << "\n1st line is " << cfgline[0].getcstr() << endl;
  */
}


bool z3950proto::init (ostream &/*logout*/) {
  // set up tcp connection to server here?
  // we might also read in the config file here (instead of librarymain.cpp)

  // 

  // logout goes to initout.txt
  //  logout <<"zdebug:init:Number of z3950 servers: "<< zserver_count << "\n";
  //logout << "\t1st server name: " << zservers[0].getName().getcstr() << "\n";
  return true;

}

/*text_t z3950proto::get_protocol_name () {
  return "z3950proto";
}
*/

void z3950proto::get_collection_list (text_tarray &collist, 
				      comerror_t &/*err*/,
				      ostream &/*logout*/) {
  // logout here DOESN'T go to initout.txt
  // logout << "zdebug: get_collection_list called:\n";

  /** *** for now, we are assuming that each SERVER is a GSDL collection,
      as opposed to each DATABASE on the servers.
  */
  z3950_server_array::iterator here = zservers.begin();
  z3950_server_array::iterator end = zservers.end();
  while (here != end) {
    collist.push_back(here->getName());
    //const ShortColInfo_t *info=here->getInfo();
    //collist.push_back(info->name);
    here++;
  }
}

void z3950proto::has_collection (const text_t &collection, bool &hascollection,
				 comerror_t &/*err*/, ostream &/*logout*/) {
  z3950_server_array::iterator here = zservers.begin();
  z3950_server_array::iterator end = zservers.end();
  while (here != end) {
    if(here->getName()==collection) {
      hascollection=true;
      return;
    }
    here++;
  }
  hascollection=false;
}

void z3950proto::ping (const text_t &/*collection*/, bool &wassuccess, 
		       comerror_t &/*err*/, ostream &/*logout*/) {
  // should we just ping the server, or actually create a connection
  // to the z39.50 server process on the machine ?
  wassuccess = true;
}

void z3950proto::get_collectinfo (const text_t &collection,
				  ColInfoResponse_t &collectinfo,
				  comerror_t &err, ostream &logout) {

  // set err to protocolError if something goes wrong...

  z3950_server_array::iterator here = zservers.begin();
  z3950_server_array::iterator end = zservers.end();
  while (here != end) {
    if(here->getName()==collection) {
      break;
    }
    here++;
  }

  // is this right? ie does end refer to the last element, or AFTER the
  // last element?
  if (here==end) {
    err=protocolError;
    logout << "z39.50: couldn't find collection" 
	   << collection.getcstr()
	   << endl;
    return;
  }

  /*  collectinfo.shortInfo.name="sdfg";
      collectinfo.shortInfo.host="localhost";
      collectinfo.shortInfo.port=0; */
  const ShortColInfo_t *colinfo=here->getInfo();
  collectinfo.shortInfo.name=colinfo->name;
  collectinfo.shortInfo.host=colinfo->host;
  collectinfo.shortInfo.port=colinfo->port;

  collectinfo.isPublic=true;
  // don't use beta field
  /*collectinfo.isBeta=false;*/
  collectinfo.buildDate=1;
  // leave ccsCols empty (no cross-coll. searching - for now)
  /*collectinfo.ccsCols=(text_tarray);*/ //not like this!!!
  collectinfo.languages.push_back("en");
  collectinfo.languages.push_back("fr");
  collectinfo.numDocs=0;
  collectinfo.numWords=0;
  collectinfo.numBytes=0;
  // copy the text maps over.
  //  collectinfo.collectionmeta; // text_tmap
  //  delete collectinfo.collectionmeta;
  collectinfo.collectionmeta=*(here->getMeta());
  /* collectinfo.format; //text_tmap
    collectinfo.building; //text_tmap
  */

collectinfo.receptionist="z3950"; /* for now... this is a url,
				    relative to .../cgi-bin   */
}

void z3950proto::get_filterinfo (const text_t &/*collection*/,
				 InfoFiltersResponse_t &/*response*/,
				 comerror_t &/*err*/, ostream &/*logout*/) {

}

void z3950proto::filter (const text_t &collection,
			 FilterRequest_t &request,
			 FilterResponse_t &response,
			 comerror_t &err, ostream &logout) {
  // assume this function is only called when creating the title page,
  // and looking for the metadata regarding icons.
  response.numDocs=0;
  response.isApprox=Approximate; // Exact | Approximate | MoreThan
  // leave response.termInfo empty
  // leave response.docInfo empty
  /* got the following for first query:
     request.filterResultOptions=FRmetadata;
     request.getParents=getParents
     request.fields=metadata
     request.docSet (includes OID);
  */
  // fill in the metadata for each of the OIDs (if it is requested)
  if (request.filterResultOptions & FRmetadata) {
    // request.fields // type text_tset
    // docInfo is type ResultDocInfo_tarray
    ColInfoResponse_t *info = new ColInfoResponse_t;
    get_collectinfo (collection, *info, err, logout);
    // should check err returned here....

    ResultDocInfo_t *docInfo=new ResultDocInfo_t;
    
    text_tmap::iterator it=info->collectionmeta.find("collectionname");
    // cerr now goes to errout.txt in etc directory
    // "(*it).first" is key, "(*it).second" is value(s).
    if (it!=info->collectionmeta.end()) 
      docInfo->metadata["collectionname"].values.push_back((*it).second);
    else // needs to exist, but be empty
      docInfo->metadata["collectionname"].values.push_back("");

    it=info->collectionmeta.find("iconcollection");
    if (it!=info->collectionmeta.end()) 
      docInfo->metadata["iconcollection"].values.push_back((*it).second);
    else // needs to exist, but be empty
      docInfo->metadata["iconcollection"].values.push_back("");

    it=info->collectionmeta.find("iconcollectionsmall");
    if (it!=info->collectionmeta.end()) 
      docInfo->metadata["iconcollectionsmall"].values.push_back((*it).second);
    else // needs to exist, but be empty
      docInfo->metadata["iconcollectionsmall"].values.push_back("");
    

    response.docInfo.push_back(*docInfo);
  }

  err=noError;
}

void z3950proto::get_document (const text_t &/*collection*/,
			       const DocumentRequest_t &/*request*/,
			       DocumentResponse_t &/*response*/,
			       comerror_t &err, ostream &logout) {

  if (0) {
    err=protocolError;
    logout << "Some error\n";
  }
}


#endif
